from itertools import count

from django.contrib.auth.models import User
from django_group_access.models import AccessGroup

from offspring.web.queuemanager.models import (
    BuildRequest,
    BuildResult,
    Lexbuilder,
    ProjectGroup,
    Project,
    Release)


class ObjectFactory(object):

    def __init__(self):
        self.counter = count()

    def getUniqueEmailAddress(self):
        return "%s@example.com" % self.getUniqueString('email')

    def getUniqueString(self, prefix=None):
        """Return a string unique to this factory instance.

        :param prefix: Used as a prefix for the unique string. If unspecified,
            defaults to 'generic-string'.
        """
        if prefix is None:
            prefix = "generic-string"
        string = "%s%s" % (prefix, self.getUniqueInteger())
        return string.lower()

    def getUniqueInteger(self):
        """Return an integer unique to this factory instance."""
        return self.counter.next()

    def makeUser(self):
        userid = password = self.getUniqueString()
        user = User.objects.create_user(
            userid, self.getUniqueEmailAddress(), password)
        user.save()
        return user

    def makeProject(self, name=None, title=None, is_private=False,
                    project_group=None, owner=None, access_groups=[]):
        if name is None:
            name = self.getUniqueString()
        if title is None:
            title = self.getUniqueString()
        if owner is None:
            owner = self.makeUser()
        project = Project(
            name=name, title=title, _is_private=is_private, owner=owner,
            project_group=project_group)
        project.save()
        for group in access_groups:
            project.access_groups.add(group)
        project.save()
        return project

    def makeProjectGroup(self, name=None, title=None):
        if name is None:
            name = self.getUniqueString()
        if title is None:
            title = self.getUniqueString()
        group = ProjectGroup(name=name, title=title)
        group.save()
        return group

    def makeBuildResult(self, project=None):
        if project is None:
            project = self.makeProject()
            project.save()
        result = BuildResult(project=project)
        result.save()
        return result

    def makeRelease(self, build=None, name=None, creator=None):
        if build is None:
            build = self.makeBuildResult()
        if name is None:
            name = self.getUniqueString()
        if creator is None:
            creator = self.makeUser()
        release = Release(build=build, name=name, creator=creator)
        release.save()
        return release

    def makeLexbuilder(self, name=None, uri=None, current_job=None):
        if name is None:
            name = self.getUniqueString()
        if uri is None:
            uri = 'htt://example.com/%s' % name
        if current_job is None:
            current_job = self.makeBuildResult()
        builder = Lexbuilder(name=name, uri=uri, current_job=current_job)
        builder.save()
        return builder

    def makeBuildRequest(self, project=None):
        if project is None:
            project = self.makeProject()
            project.save()
        request = BuildRequest(project=project)
        request.save()
        return request

    def makeAccessGroup(self, users):
        name = self.getUniqueString()
        group = AccessGroup(name=name)
        group.save()
        for user in users:
            group.members.add(user)
        group.save()
        return group


factory = ObjectFactory()
